@@ -1,6 +1,7 @@
 /*
  * libalsa output driver. This file is part of Shairport.
  * Copyright (c) Muffinman, Skaman 2013
+ * Copyright (c) Mike Brady 2014 -- 2018
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person
@@ -39,7 +40,7 @@ static void help(void);
 static int init(int argc, char **argv);
 static void deinit(void);
 static void start(int i_sample_rate, int i_sample_format);
-static void play(short buf[], int samples);
+static void play(void *buf, int samples);
 static void stop(void);
 static void flush(void);
 int delay(long *the_delay);
@@ -88,15 +89,22 @@ static char *alsa_mix_ctrl = "Master";
 static int alsa_mix_index = 0;
 static int hardware_mixer = 0;
 static int has_softvol = 0;
+
 static int volume_set_request = 0;       // set when an external request is made to set the volume.
 int mute_request_pending = 0;            //  set when an external request is made to mute or unmute.
 int overriding_mute_state_requested = 0; // 1 = mute; 0 = unmute requested
+int mixer_volume_setting_gives_mute =
+    0;              // set when it is discovered that particular mixer volume setting causes a mute.
+long alsa_mix_mute; // setting the volume to this value mutes output, if
+                    // mixer_volume_setting_gives_mute is true
+int volume_based_mute_is_active =
+    0; // set when muting is being done by a setting the volume to a magic value
 
 static snd_pcm_sframes_t (*alsa_pcm_write)(snd_pcm_t *, const void *,
                                            snd_pcm_uframes_t) = snd_pcm_writei;
 
-static int play_number;
-static int64_t accumulated_delay, accumulated_da_delay;
+// static int play_number;
+// static int64_t accumulated_delay, accumulated_da_delay;
 int alsa_characteristics_already_listed = 0;
 
 static snd_pcm_uframes_t period_size_requested, buffer_size_requested;
@@ -114,7 +122,7 @@ void set_alsa_out_dev(char *dev) { alsa_out_dev = dev; }
 
 int open_mixer() {
   if (hardware_mixer) {
-    debug(2, "Open Mixer");
+    debug(3, "Open Mixer");
     int ret = 0;
     snd_mixer_selem_id_alloca(&alsa_mix_sid);
     snd_mixer_selem_id_set_index(alsa_mix_sid, alsa_mix_index);
@@ -148,21 +156,26 @@ void close_mixer() {
   }
 }
 
-#define RELEASE_ALSA_MUTEX_AND_DIE(X)                                                              \
-  pthread_mutex_unlock(&alsa_mutex);                                                               \
-  die(X);
+void do_snd_mixer_selem_set_playback_dB_all(snd_mixer_elem_t *mix_elem, double vol) {
+  if (snd_mixer_selem_set_playback_dB_all(mix_elem, vol, 0) != 0) {
+    debug(1, "Can't set playback volume accurately to %f dB.", vol);
+    if (snd_mixer_selem_set_playback_dB_all(mix_elem, vol, -1) != 0)
+      if (snd_mixer_selem_set_playback_dB_all(mix_elem, vol, 1) != 0)
+        debug(1, "Could not set playback dB volume on the mixer.");
+  }
+}
 
 static int init(int argc, char **argv) {
-  pthread_mutex_lock(&alsa_mutex);
+  debug_mutex_lock(&alsa_mutex, 1000, 1);
   // debug(2,"audio_alsa init called.");
   const char *str;
   int value;
-  double dvalue;
+  // double dvalue;
 
   // set up default values first
   set_period_size_request = 0;
   set_buffer_size_request = 0;
-  config.alsa_use_playback_switch_for_mute = 0; // don't use it by default
+  config.alsa_use_hardware_mute = 0; // don't use it by default
 
   config.audio_backend_latency_offset = 0;
   config.audio_backend_buffer_desired_length = 0.15;
@@ -206,20 +219,35 @@ static int init(int argc, char **argv) {
       else if (strcasecmp(str, "yes") == 0)
         config.no_sync = 1;
       else {
-        pthread_mutex_unlock(&alsa_mutex);
+        debug_mutex_unlock(&alsa_mutex, 3);
         die("Invalid disable_synchronization option choice \"%s\". It should be \"yes\" or \"no\"");
       }
     }
 
     /* Get the mute_using_playback_switch setting. */
     if (config_lookup_string(config.cfg, "alsa.mute_using_playback_switch", &str)) {
+      inform("The alsa \"mute_using_playback_switch\" setting is deprecated. "
+             "Please use the \"use_hardware_mute_if_available\" setting instead.");
       if (strcasecmp(str, "no") == 0)
-        config.alsa_use_playback_switch_for_mute = 0;
+        config.alsa_use_hardware_mute = 0;
       else if (strcasecmp(str, "yes") == 0)
-        config.alsa_use_playback_switch_for_mute = 1;
+        config.alsa_use_hardware_mute = 1;
       else {
-        pthread_mutex_unlock(&alsa_mutex);
-        die("Invalid mute_use_playback_switch option choice \"%s\". It should be \"yes\" or "
+        debug_mutex_unlock(&alsa_mutex, 3);
+        die("Invalid mute_using_playback_switch option choice \"%s\". It should be \"yes\" or "
+            "\"no\"");
+      }
+    }
+
+    /* Get the use_hardware_mute_if_available setting. */
+    if (config_lookup_string(config.cfg, "alsa.use_hardware_mute_if_available", &str)) {
+      if (strcasecmp(str, "no") == 0)
+        config.alsa_use_hardware_mute = 0;
+      else if (strcasecmp(str, "yes") == 0)
+        config.alsa_use_hardware_mute = 1;
+      else {
+        debug_mutex_unlock(&alsa_mutex, 3);
+        die("Invalid use_hardware_mute_if_available option choice \"%s\". It should be \"yes\" or "
             "\"no\"");
       }
     }
@@ -241,7 +269,7 @@ static int init(int argc, char **argv) {
       else if (strcasecmp(str, "S8") == 0)
         config.output_format = SPS_FORMAT_S8;
       else {
-        pthread_mutex_unlock(&alsa_mutex);
+        debug_mutex_unlock(&alsa_mutex, 3);
         die("Invalid output format \"%s\". It should be \"U8\", \"S8\", \"S16\", \"S24\", "
             "\"S24_3LE\", \"S24_3BE\" or "
             "\"S32\"",
@@ -251,7 +279,7 @@ static int init(int argc, char **argv) {
 
     /* Get the output rate, which must be a multiple of 44,100*/
     if (config_lookup_int(config.cfg, "alsa.output_rate", &value)) {
-      debug(1, "Value read for output rate is %d.", value);
+      debug(1, "alsa output rate is %d frames per second", value);
       switch (value) {
       case 44100:
       case 88200:
@@ -260,7 +288,7 @@ static int init(int argc, char **argv) {
         config.output_rate = value;
         break;
       default:
-        pthread_mutex_unlock(&alsa_mutex);
+        debug_mutex_unlock(&alsa_mutex, 3);
         die("Invalid output rate \"%d\". It should be a multiple of 44,100 up to 352,800", value);
       }
     }
@@ -272,7 +300,7 @@ static int init(int argc, char **argv) {
       else if (strcasecmp(str, "yes") == 0)
         config.no_mmap = 0;
       else {
-        pthread_mutex_unlock(&alsa_mutex);
+        debug_mutex_unlock(&alsa_mutex, 3);
         die("Invalid use_mmap_if_available option choice \"%s\". It should be \"yes\" or \"no\"");
       }
     }
@@ -281,7 +309,7 @@ static int init(int argc, char **argv) {
       set_period_size_request = 1;
       debug(1, "Value read for period size is %d.", value);
       if (value < 0) {
-        pthread_mutex_unlock(&alsa_mutex);
+        debug_mutex_unlock(&alsa_mutex, 3);
         die("Invalid alsa period size setting \"%d\". It "
             "must be greater than 0.",
             value);
@@ -295,7 +323,7 @@ static int init(int argc, char **argv) {
       set_buffer_size_request = 1;
       debug(1, "Value read for buffer size is %d.", value);
       if (value < 0) {
-        pthread_mutex_unlock(&alsa_mutex);
+        debug_mutex_unlock(&alsa_mutex, 3);
         die("Invalid alsa buffer size setting \"%d\". It "
             "must be greater than 0.",
             value);
@@ -334,13 +362,13 @@ static int init(int argc, char **argv) {
       break;
     default:
       help();
-      pthread_mutex_unlock(&alsa_mutex);
+      debug_mutex_unlock(&alsa_mutex, 3);
       die("Invalid audio option -%c specified", opt);
     }
   }
 
   if (optind < argc) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     die("Invalid audio argument: %s", argv[optind]);
   }
 
@@ -365,8 +393,12 @@ static int init(int argc, char **argv) {
         audio_alsa.volume = &volume; // insert the volume function now we know it can do dB stuff
         audio_alsa.parameters = &parameters; // likewise the parameters stuff
         if (alsa_mix_mindb == SND_CTL_TLV_DB_GAIN_MUTE) {
-          // Raspberry Pi does this
-          debug(1, "Lowest dB value is a mute -- try minimum volume +1");
+          // For instance, the Raspberry Pi does this
+          debug(1, "Lowest dB value is a mute");
+          mixer_volume_setting_gives_mute = 1;
+          alsa_mix_mute = SND_CTL_TLV_DB_GAIN_MUTE; // this may not be necessary -- it's always
+                                                    // going to be SND_CTL_TLV_DB_GAIN_MUTE, right?
+          // debug(1, "Try minimum volume + 1 as lowest true attenuation value");
           if (snd_mixer_selem_ask_playback_vol_dB(alsa_mix_elem, alsa_mix_minv + 1,
                                                   &alsa_mix_mindb) != 0)
             debug(1, "Can't get dB value corresponding to a minimum volume + 1.");
@@ -380,11 +412,11 @@ static int init(int argc, char **argv) {
               alsa_mix_ctrl);
 
         if (snd_ctl_open(&ctl, alsa_mix_dev, 0) < 0) {
-          pthread_mutex_unlock(&alsa_mutex);
+          debug_mutex_unlock(&alsa_mutex, 3);
           die("Cannot open control \"%s\"", alsa_mix_dev);
         }
         if (snd_ctl_elem_id_malloc(&elem_id) < 0) {
-          pthread_mutex_unlock(&alsa_mutex);
+          debug_mutex_unlock(&alsa_mutex, 3);
           die("Cannot allocate memory for control \"%s\"", alsa_mix_dev);
         }
         snd_ctl_elem_id_set_interface(elem_id, SND_CTL_ELEM_IFACE_MIXER);
@@ -416,8 +448,9 @@ static int init(int argc, char **argv) {
         */
       }
     }
-    if ((config.alsa_use_playback_switch_for_mute == 1) &&
-        (snd_mixer_selem_has_playback_switch(alsa_mix_elem))) {
+    if (((config.alsa_use_hardware_mute == 1) &&
+         (snd_mixer_selem_has_playback_switch(alsa_mix_elem))) ||
+        mixer_volume_setting_gives_mute) {
       audio_alsa.mute = &mute; // insert the mute function now we know it can do muting stuff
       // debug(1, "Has mixer and mute ability we will use.");
     } else {
@@ -429,7 +462,7 @@ static int init(int argc, char **argv) {
   }
 
   alsa_mix_handle = NULL;
-  pthread_mutex_unlock(&alsa_mutex);
+  debug_mutex_unlock(&alsa_mutex, 3);
   return 0;
 }
 
@@ -444,13 +477,16 @@ int open_alsa_device(void) {
   const snd_pcm_uframes_t minimal_buffer_headroom =
       352 * 2; // we accept this much headroom in the hardware buffer, but we'll
                // accept less
-  const snd_pcm_uframes_t requested_buffer_headroom =
-      minimal_buffer_headroom + 2048; // we ask for this much headroom in the
-                                      // hardware buffer, but we'll accept less
+               /*
+                 const snd_pcm_uframes_t requested_buffer_headroom =
+                     minimal_buffer_headroom + 2048; // we ask for this much headroom in the
+                                                     // hardware buffer, but we'll accept less
+               */
+
   int ret, dir = 0;
   unsigned int my_sample_rate = desired_sample_rate;
   // snd_pcm_uframes_t frames = 441 * 10;
-  snd_pcm_uframes_t buffer_size, actual_buffer_length;
+  snd_pcm_uframes_t actual_buffer_length;
   snd_pcm_access_t access;
 
   // ensure no calls are made to the alsa device enquiring about the buffer length if
@@ -471,7 +507,7 @@ int open_alsa_device(void) {
 
   ret = snd_pcm_hw_params_any(alsa_handle, alsa_params);
   if (ret < 0) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     ;
     die("audio_alsa: Broken configuration for device \"%s\": no configurations "
         "available",
@@ -482,14 +518,14 @@ int open_alsa_device(void) {
       (snd_pcm_hw_params_set_access(alsa_handle, alsa_params, SND_PCM_ACCESS_MMAP_INTERLEAVED) >=
        0)) {
     if (output_method_signalled == 0) {
-      debug(1, "Output written using MMAP");
+      debug(3, "Output written using MMAP");
       output_method_signalled = 1;
     }
     access = SND_PCM_ACCESS_MMAP_INTERLEAVED;
     alsa_pcm_write = snd_pcm_mmap_writei;
   } else {
     if (output_method_signalled == 0) {
-      debug(1, "Output written with RW");
+      debug(3, "Output written with RW");
       output_method_signalled = 1;
     }
     access = SND_PCM_ACCESS_RW_INTERLEAVED;
@@ -498,7 +534,7 @@ int open_alsa_device(void) {
 
   ret = snd_pcm_hw_params_set_access(alsa_handle, alsa_params, access);
   if (ret < 0) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     die("audio_alsa: Access type not available for device \"%s\": %s", alsa_out_dev,
         snd_strerror(ret));
   }
@@ -526,19 +562,20 @@ int open_alsa_device(void) {
     sf = SND_PCM_FORMAT_S32;
     break;
   default:
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
+    sf = SND_PCM_FORMAT_S16; // this is just to quieten a compiler warning
     die("Unsupported output format at audio_alsa.c");
   }
   ret = snd_pcm_hw_params_set_format(alsa_handle, alsa_params, sf);
   if (ret < 0) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     die("audio_alsa: Sample format %d not available for device \"%s\": %s", sample_format,
         alsa_out_dev, snd_strerror(ret));
   }
 
   ret = snd_pcm_hw_params_set_channels(alsa_handle, alsa_params, 2);
   if (ret < 0) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     die("audio_alsa: Channels count (2) not available for device \"%s\": %s", alsa_out_dev,
         snd_strerror(ret));
   }
@@ -554,7 +591,7 @@ int open_alsa_device(void) {
     ret = snd_pcm_hw_params_set_period_size_near(alsa_handle, alsa_params, &period_size_requested,
                                                  &dir);
     if (ret < 0) {
-      pthread_mutex_unlock(&alsa_mutex);
+      debug_mutex_unlock(&alsa_mutex, 3);
       die("audio_alsa: cannot set period size of %lu: %s", period_size_requested,
           snd_strerror(ret));
       snd_pcm_uframes_t actual_period_size;
@@ -570,33 +607,33 @@ int open_alsa_device(void) {
     debug(1, "Attempting to set the buffer size to %lu", buffer_size_requested);
     ret = snd_pcm_hw_params_set_buffer_size_near(alsa_handle, alsa_params, &buffer_size_requested);
     if (ret < 0) {
-      pthread_mutex_unlock(&alsa_mutex);
+      debug_mutex_unlock(&alsa_mutex, 3);
       die("audio_alsa: cannot set buffer size of %lu: %s", buffer_size_requested,
           snd_strerror(ret));
-      snd_pcm_uframes_t actual_buffer_size;
-      snd_pcm_hw_params_get_buffer_size(alsa_params, &actual_buffer_size);
-      if (actual_buffer_size != buffer_size_requested)
-        inform("Actual period size set to a different value than requested. Requested: %lu, actual "
-               "setting: %lu",
-               buffer_size, actual_buffer_size);
     }
+    snd_pcm_uframes_t actual_buffer_size;
+    snd_pcm_hw_params_get_buffer_size(alsa_params, &actual_buffer_size);
+    if (actual_buffer_size != buffer_size_requested)
+      inform("Actual period size set to a different value than requested. Requested: %lu, actual "
+             "setting: %lu",
+             buffer_size_requested, actual_buffer_size);
   }
 
   ret = snd_pcm_hw_params(alsa_handle, alsa_params);
   if (ret < 0) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     die("audio_alsa: Unable to set hw parameters for device \"%s\": %s.", alsa_out_dev,
         snd_strerror(ret));
   }
 
   if (my_sample_rate != desired_sample_rate) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     die("Can't set the D/A converter to %d.", desired_sample_rate);
   }
 
   ret = snd_pcm_hw_params_get_buffer_size(alsa_params, &actual_buffer_length);
   if (ret < 0) {
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     die("audio_alsa: Unable to get hw buffer length for device \"%s\": %s.", alsa_out_dev,
         snd_strerror(ret));
   }
@@ -632,11 +669,11 @@ int open_alsa_device(void) {
   if (alsa_characteristics_already_listed == 0) {
     alsa_characteristics_already_listed = 1;
     int log_level = 2; // the level at which debug information should be output
-    int rc;
+                       //    int rc;
     snd_pcm_access_t access_type;
     snd_pcm_format_t format_type;
     snd_pcm_subformat_t subformat_type;
-    unsigned int val, val2;
+    //    unsigned int val, val2;
     unsigned int uval, uval2;
     int sval;
     int dir;
@@ -780,15 +817,15 @@ int delay(long *the_delay) {
   if (alsa_handle == NULL) {
     return -ENODEV;
   } else {
-    pthread_mutex_lock(&alsa_mutex);
-    int derr, ignore;
+    debug_mutex_lock(&alsa_mutex, 10000, 1);
+    int derr;
     if (snd_pcm_state(alsa_handle) == SND_PCM_STATE_RUNNING) {
       *the_delay = 0; // just to see what happens
       reply = snd_pcm_delay(alsa_handle, the_delay);
       if (reply != 0) {
         debug(1, "Error %d in delay(): \"%s\". Delay reported is %d frames.", reply,
               snd_strerror(reply), *the_delay);
-        ignore = snd_pcm_recover(alsa_handle, reply, 1);
+        snd_pcm_recover(alsa_handle, reply, 1);
       }
     } else if (snd_pcm_state(alsa_handle) == SND_PCM_STATE_PREPARED) {
       *the_delay = 0;
@@ -802,11 +839,11 @@ int delay(long *the_delay) {
         debug(1, "Error -- ALSA delay(): bad state: %d.", snd_pcm_state(alsa_handle));
       }
       if ((derr = snd_pcm_prepare(alsa_handle))) {
-        ignore = snd_pcm_recover(alsa_handle, derr, 1);
+        snd_pcm_recover(alsa_handle, derr, 1);
         debug(1, "Error preparing after delay error: \"%s\".", snd_strerror(derr));
       }
     }
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
     // here, occasionally pretend there's a problem with pcm_get_delay()
     // if ((random() % 100000) < 3) // keep it pretty rare
     //	reply = -EPERM; // pretend something bad has happened
@@ -814,11 +851,11 @@ int delay(long *the_delay) {
   }
 }
 
-static void play(short buf[], int samples) {
+static void play(void *buf, int samples) {
   // debug(3,"audio_alsa play called.");
   int ret = 0;
   if (alsa_handle == NULL) {
-    pthread_mutex_lock(&alsa_mutex);
+    debug_mutex_lock(&alsa_mutex, 10000, 1);
     ret = open_alsa_device();
     if (ret == 0) {
       if (audio_alsa.volume)
@@ -826,15 +863,15 @@ static void play(short buf[], int samples) {
       if (audio_alsa.mute)
         do_mute(0);
     }
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
   }
   if (ret == 0) {
-    pthread_mutex_lock(&alsa_mutex);
-    snd_pcm_sframes_t current_delay = 0;
-    int err, ignore;
+    debug_mutex_lock(&alsa_mutex, 10000, 1);
+    //    snd_pcm_sframes_t current_delay = 0;
+    int err;
     if (snd_pcm_state(alsa_handle) == SND_PCM_STATE_XRUN) {
       if ((err = snd_pcm_prepare(alsa_handle))) {
-        ignore = snd_pcm_recover(alsa_handle, err, 1);
+        snd_pcm_recover(alsa_handle, err, 1);
         debug(1, "Error preparing after underrun: \"%s\".", snd_strerror(err));
       }
     }
@@ -845,71 +882,45 @@ static void play(short buf[], int samples) {
       if (samples == 0)
         debug(1, "empty buffer being passed to pcm_writei -- skipping it");
       if ((samples != 0) && (buf != NULL)) {
-        err = alsa_pcm_write(alsa_handle, (char *)buf, samples);
+        err = alsa_pcm_write(alsa_handle, buf, samples);
         if (err < 0) {
           debug(1, "Error %d writing %d samples in play(): \"%s\".", err, samples,
                 snd_strerror(err));
-          ignore = snd_pcm_recover(alsa_handle, err, 1);
+          snd_pcm_recover(alsa_handle, err, 1);
         }
       }
     } else {
       debug(1, "Error -- ALSA device in incorrect state (%d) for play.",
             snd_pcm_state(alsa_handle));
       if ((err = snd_pcm_prepare(alsa_handle))) {
-        ignore = snd_pcm_recover(alsa_handle, err, 1);
+        snd_pcm_recover(alsa_handle, err, 1);
         debug(1, "Error preparing after play error: \"%s\".", snd_strerror(err));
       }
     }
-    pthread_mutex_unlock(&alsa_mutex);
+    debug_mutex_unlock(&alsa_mutex, 3);
   }
 }
 
 static void flush(void) {
   // debug(2,"audio_alsa flush called.");
-  pthread_mutex_lock(&alsa_mutex);
+  debug_mutex_lock(&alsa_mutex, 10000, 1);
   int derr;
   do_mute(1);
   if (alsa_handle) {
-    // debug(1,"Dropping frames for flush...");
-    /*
+
     if ((derr = snd_pcm_drop(alsa_handle)))
-      debug(1, "Error dropping frames: \"%s\".", snd_strerror(derr));
-    // debug(1,"Dropped frames ok. State is %d.",snd_pcm_state(alsa_handle));
-    if ((derr = snd_pcm_prepare(alsa_handle)))
-      debug(1, "Error preparing after flush: \"%s\".", snd_strerror(derr));
-    // debug(1,"Frames successfully dropped.");
-    */
-
-    /*
-    if (snd_pcm_state(alsa_handle)==SND_PCM_STATE_PREPARED)
-      debug(1,"Flush returns to SND_PCM_STATE_PREPARED state.");
-    if (snd_pcm_state(alsa_handle)==SND_PCM_STATE_RUNNING)
-      debug(1,"Flush returns to SND_PCM_STATE_RUNNING state.");
-    */
-    /*
-    if (!((snd_pcm_state(alsa_handle) == SND_PCM_STATE_PREPARED) ||
-          (snd_pcm_state(alsa_handle) == SND_PCM_STATE_RUNNING)))
-      debug(1, "Flush returning unexpected state -- %d.", snd_pcm_state(alsa_handle));
-    */
-
-    // this is derived from
-    // http://www.alsa-project.org/alsa-doc/alsa-lib/_2test_2latency_8c-example.html#a45
-
-    if ((derr = snd_pcm_nonblock(alsa_handle, 0)))
-      debug(1, "Error %d (\"%s\") unblocking output device.", derr, snd_strerror(derr));
-    if ((derr = snd_pcm_drain(alsa_handle)))
-      debug(1, "Error %d (\"%s\") draining output device.", derr, snd_strerror(derr));
-    if ((derr = snd_pcm_nonblock(alsa_handle, 1)))
-      debug(1, "Error %d (\"%s\") reblocking output device.", derr, snd_strerror(derr));
+      debug(1, "Error %d (\"%s\") dropping output device.", derr, snd_strerror(derr));
 
     if ((derr = snd_pcm_hw_free(alsa_handle)))
-      debug(1, "Error %d (\"%s\") freeing output device hardware.", derr, snd_strerror(derr));
+      debug(1, "Error %d (\"%s\") freeing the output device hardware.", derr, snd_strerror(derr));
 
     // flush also closes the device
-    snd_pcm_close(alsa_handle);
+    if ((derr = snd_pcm_close(alsa_handle)))
+      debug(1, "Error %d (\"%s\") closing the output device.", derr, snd_strerror(derr));
+
     alsa_handle = NULL;
   }
-  pthread_mutex_unlock(&alsa_mutex);
+  debug_mutex_unlock(&alsa_mutex, 3);
 }
 
 static void stop(void) {
@@ -949,24 +960,23 @@ void do_volume(double vol) { // caller is assumed to have the alsa_mutex when us
                    "control.");
       }
     } else {
-      if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, 0) != 0) {
-        debug(1, "Can't set playback volume accurately to %f dB.", vol);
-        if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, -1) != 0)
-          if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, 1) != 0)
-            debug(1, "Could not set playback dB volume on the mixer.");
+      if (volume_based_mute_is_active == 0) {
+        // debug(1,"Set alsa volume.");
+        do_snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol);
+      } else {
+        debug(2, "Not setting volume because volume-based mute is active");
       }
     }
     volume_set_request = 0; // any external request that has been made is now satisfied
-    // debug(1,"Alsa volume actually set.");
     close_mixer();
   }
 }
 
 void volume(double vol) {
-  pthread_mutex_lock(&alsa_mutex);
+  debug_mutex_lock(&alsa_mutex, 1000, 1);
   volume_set_request = 1; // an external request has been made to set the volume
   do_volume(vol);
-  pthread_mutex_unlock(&alsa_mutex);
+  debug_mutex_unlock(&alsa_mutex, 3);
 }
 
 /*
@@ -989,11 +999,11 @@ static void linear_volume(double vol) {
 
 static void mute(int mute_state_requested) {
   // debug(1,"External Mute Request: %d",mute_state_requested);
-  pthread_mutex_lock(&alsa_mutex);
+  debug_mutex_lock(&alsa_mutex, 10000, 1);
   mute_request_pending = 1;
   overriding_mute_state_requested = mute_state_requested;
   do_mute(mute_state_requested);
-  pthread_mutex_unlock(&alsa_mutex);
+  debug_mutex_unlock(&alsa_mutex, 3);
 }
 
 void do_mute(int mute_state_requested) {
@@ -1011,16 +1021,28 @@ void do_mute(int mute_state_requested) {
   // If the hardware isn't there, or we are not allowed to use it, nothing will be done
   // The caller must have the alsa mutex
 
-  if (config.alsa_use_playback_switch_for_mute == 1) {
+  if (config.alsa_use_hardware_mute == 1) {
     if (mute_request_pending == 0)
       local_mute_state_requested = mute_state_requested;
     if (open_mixer()) {
       if (local_mute_state_requested) {
         // debug(1,"Playback Switch mute actually done");
-        snd_mixer_selem_set_playback_switch_all(alsa_mix_elem, 0);
+        if (snd_mixer_selem_has_playback_switch(alsa_mix_elem))
+          snd_mixer_selem_set_playback_switch_all(alsa_mix_elem, 0);
+        else {
+          // debug(1,"Activating volume-based mute.");
+          volume_based_mute_is_active = 1;
+          do_snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, alsa_mix_mute);
+        }
       } else if (overriding_mute_state_requested == 0) {
         // debug(1,"Playback Switch unmute actually done");
-        snd_mixer_selem_set_playback_switch_all(alsa_mix_elem, 1);
+        if (snd_mixer_selem_has_playback_switch(alsa_mix_elem))
+          snd_mixer_selem_set_playback_switch_all(alsa_mix_elem, 1);
+        else {
+          // debug(1,"Deactivating volume-based mute.");
+          volume_based_mute_is_active = 0;
+          do_snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, set_volume);
+        }
       }
       close_mixer();
     }
