@@ -37,18 +37,18 @@
 int Fs;
 long long starttime, samples_played;
 
-static int init(int argc, char **argv) { return 0; }
+static int init(__attribute__((unused)) int argc, __attribute__((unused)) char **argv) { return 0; }
 
 static void deinit(void) {}
 
-static void start(int sample_rate, int sample_format) {
+static void start(int sample_rate, __attribute__((unused)) int sample_format) {
   Fs = sample_rate;
   starttime = 0;
   samples_played = 0;
   debug(1, "dummy audio output started at Fs=%d Hz\n", sample_rate);
 }
 
-static void play(short buf[], int samples) {}
+static void play(__attribute__((unused)) void *buf, __attribute__((unused)) int samples) {}
 
 static void stop(void) { debug(1, "dummy audio stopped\n"); }
 
