@@ -5,26 +5,14 @@
 #include <signal.h>
 #include <stdint.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include "audio.h"
 #include "config.h"
 #include "definitions.h"
 #include "mdns.h"
 
-#if defined(__APPLE__) && defined(__MACH__)
-/* Apple OSX and iOS (Darwin). ------------------------------ */
-#include <TargetConditionals.h>
-#if TARGET_OS_MAC == 1
-/* OSX */
-#define COMPILE_FOR_OSX 1
-#endif
-#endif
-
-#if defined(__linux__) || defined(__FreeBSD__) || defined(__CYGWIN__)
-/* Linux and FreeBSD */
-#define COMPILE_FOR_LINUX_AND_FREEBSD_AND_CYGWIN 1
-#endif
-
 // struct sockaddr_in6 is bigger than struct sockaddr. derp
 #ifdef AF_INET6
 #define SOCKADDR struct sockaddr_storage
@@ -60,6 +48,11 @@ enum playback_mode_type {
   ST_right_only,
 } playback_mode_type;
 
+enum volume_control_profile_type {
+  VCP_standard = 0,
+  VCP_flat,
+} volume_control_profile_type;
+
 enum decoders_supported_type {
   decoder_hammerton = 0,
   decoder_apple_alac,
@@ -82,7 +75,7 @@ enum sps_format_t {
 typedef struct {
   config_t *cfg;
   double airplay_volume; // stored here for reloading when necessary
-  char *appName; // normally the app is called shairport-syn, but it may be symlinked
+  char *appName;         // normally the app is called shairport-syn, but it may be symlinked
   char *password;
   char *service_name; // the name for the shairport service, e.g. "Shairport Sync Version %v running
                       // on host %h"
@@ -96,7 +89,7 @@ typedef struct {
   char *metadata_pipename;
   char *metadata_sockaddr;
   int metadata_sockport;
-  int metadata_sockmsglength;
+  size_t metadata_sockmsglength;
   int get_coverart;
 #endif
   uint8_t hw_addr[6];
@@ -121,16 +114,18 @@ typedef struct {
   char *mdns_name;
   mdns_backend *mdns;
   int buffer_start_fill;
-  int64_t latency;
   int64_t userSuppliedLatency; // overrides all other latencies -- use with caution
-  int64_t iTunesLatency;       // supplied with --iTunesLatency option
-  int64_t AirPlayLatency;      // supplied with --AirPlayLatency option
-  int64_t ForkedDaapdLatency;  // supplied with --ForkedDaapdLatency option
+  int64_t fixedLatencyOffset;  // add this to all automatic latencies supplied to get the actual
+                               // total latency
+  // the total latency will be limited to the min and max-latency values, if supplied
   int daemonise;
   int daemonise_store_pid; // don't try to save a PID file
   char *piddir;
   char *computed_piddir; // the actual pid directory to create, if any
-  int logOutputLevel;    // log output level
+
+  int logOutputLevel;              // log output level
+  int debugger_show_elapsed_time;  // in the debug message, display the time since startup
+  int debugger_show_relative_time; // in the debug message, display the time since the last one
   int statistics_requested, use_negotiated_latencies;
   enum playback_mode_type playback_mode;
   char *cmd_start, *cmd_stop, *cmd_set_volume;
@@ -156,6 +151,7 @@ typedef struct {
   uint32_t volume_range_db; // the range, in dB, from max dB to min dB. Zero means use the mixer's
                             // native range.
   enum sps_format_t output_format;
+  enum volume_control_profile_type volume_control_profile;
   int output_rate;
 
 #ifdef CONFIG_CONVOLUTION
@@ -167,7 +163,7 @@ typedef struct {
 
   int loudness;
   float loudness_reference_volume_db;
-  int alsa_use_playback_switch_for_mute;
+  int alsa_use_hardware_mute;
 #if defined(HAVE_DBUS)
   enum dbus_session_type dbus_service_bus_type;
 #endif
@@ -175,8 +171,27 @@ typedef struct {
   enum dbus_session_type mpris_service_bus_type;
 #endif
 
+#ifdef HAVE_METADATA_HUB
+  char *cover_art_cache_dir;
+  int scan_interval_when_active;   // number of seconds between DACP server scans when playing
+                                   // something (1)
+  int scan_interval_when_inactive; // number of seconds between DACP server scans playing nothing
+                                   // (3)
+  int scan_max_bad_response_count; // number of successive bad results to ignore before giving up
+                                   // (10)
+  int scan_max_inactive_count;     // number of scans to do before stopping if not made active again
+                                   // (about 15 minutes worth)
+#endif
+  int disable_resend_requests; // set this to stop resend request being made for missing packets
+  double diagnostic_drop_packet_fraction; // pseudo randomly drop this fraction of packets, for
+                                          // debugging. Currently audio packets only...
 } shairport_cfg;
 
+uint32_t nctohl(const uint8_t *p); // read 4 characters from *p and do ntohl on them
+uint16_t nctohs(const uint8_t *p); // read 2 characters from *p and do ntohs on them
+
+void memory_barrier();
+
 // true if Shairport Sync is supposed to be sending output to the output device, false otherwise
 
 int get_requested_connection_state_to_output();
@@ -213,14 +228,20 @@ char *base64_enc(uint8_t *input, int length);
 #define RSA_MODE_KEY (1)
 uint8_t *rsa_apply(uint8_t *input, int inlen, int *outlen, int mode);
 
+// given a volume (0 to -30) and high and low attenuations in dB*100 (e.g. 0 to -6000 for 0 to -60
+// dB), return an attenuation depending on a linear interpolation along along the range
+double flat_vol2attn(double vol, long max_db, long min_db);
+
 // given a volume (0 to -30) and high and low attenuations in dB*100 (e.g. 0 to -6000 for 0 to -60
 // dB), return an attenuation depending on the transfer function
 double vol2attn(double vol, long max_db, long min_db);
 
 // return a monolithic (always increasing) time in nanoseconds
-
 uint64_t get_absolute_time_in_fp(void);
 
+// time at startup for debugging timing
+uint64_t fp_time_at_startup, fp_time_at_last_debug_message;
+
 // this is for reading an unsigned 32 bit number, such as an RTP timestamp
 
 long endianness;
@@ -233,9 +254,31 @@ void command_start(void);
 void command_stop(void);
 void command_set_volume(double volume);
 
+int mkpath(const char *path, mode_t mode);
+
 void shairport_shutdown();
 // void shairport_startup_complete(void);
 
 extern sigset_t pselect_sigset;
 
+// wait for the specified time in microseconds -- it checks every 20 milliseconds
+int sps_pthread_mutex_timedlock(pthread_mutex_t *mutex, useconds_t dally_time,
+                                const char *debugmessage, int debuglevel);
+// wait for the specified time, checking every 20 milliseconds, and block if it can't acquire the
+// lock
+int _debug_mutex_lock(pthread_mutex_t *mutex, useconds_t dally_time, const char *filename,
+                      const int line, int debuglevel);
+
+#define debug_mutex_lock(mu, t, d) _debug_mutex_lock(mu, t, __FILE__, __LINE__, d)
+
+int _debug_mutex_unlock(pthread_mutex_t *mutex, const char *filename, const int line,
+                        int debuglevel);
+
+#define debug_mutex_unlock(mu, d) _debug_mutex_unlock(mu, __FILE__, __LINE__, d)
+
+char *get_version_string(); // mallocs a string space -- remember to free it afterwards
+
+void sps_nanosleep(const time_t sec,
+                   const long nanosec); // waits for this time, even through interruptions
+
 #endif // _COMMON_H
