@@ -1,7 +1,7 @@
 /*
  * Embedded Avahi client. This file is part of Shairport.
  * Copyright (c) James Laird 2013
- * Additions for metadata and for detecting IPv6 Copyright (c) Mike Brady 2015
+ * Additions for metadata and for detecting IPv6 Copyright (c) Mike Brady 2015--2018
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person
@@ -33,7 +33,7 @@
 #include "common.h"
 #include "mdns.h"
 #include "rtsp.h"
-#ifdef CONFIG_DACP
+#ifdef HAVE_DACP_CLIENT
 #include "dacp.h"
 #endif
 #include <string.h>
@@ -47,10 +47,18 @@
 #include <avahi-client/lookup.h>
 #include <avahi-common/alternative.h>
 
+#define check_avahi_response(debugLevelArg, veryUnLikelyArgumentName)                              \
+  {                                                                                                \
+    int rc = veryUnLikelyArgumentName;                                                             \
+    if (rc)                                                                                        \
+      debug(debugLevelArg, "avahi call response %d at __FILE__, __LINE__)", rc);                   \
+  }
+
 typedef struct {
   AvahiThreadedPoll *service_poll;
   AvahiClient *service_client;
   AvahiServiceBrowser *service_browser;
+  char *dacp_id;
 } dacp_browser_struct;
 
 // static AvahiServiceBrowser *sb = NULL;
@@ -66,53 +74,51 @@ static int port = 0;
 static void resolve_callback(AvahiServiceResolver *r, AVAHI_GCC_UNUSED AvahiIfIndex interface,
                              AVAHI_GCC_UNUSED AvahiProtocol protocol, AvahiResolverEvent event,
                              const char *name, const char *type, const char *domain,
-                             const char *host_name, const AvahiAddress *address, uint16_t port,
-                             AvahiStringList *txt, AvahiLookupResultFlags flags, void *userdata) {
+                             __attribute__((unused)) const char *host_name,
+                             __attribute__((unused)) const AvahiAddress *address, uint16_t port,
+                             __attribute__((unused)) AvahiStringList *txt,
+                             __attribute__((unused)) AvahiLookupResultFlags flags, void *userdata) {
   assert(r);
-  
-  rtsp_conn_info *conn = (rtsp_conn_info *)userdata;
-  dacp_browser_struct *dbs = (dacp_browser_struct *)conn->mdns_private_pointer;
+
+  dacp_browser_struct *dbs = (dacp_browser_struct *)userdata;
 
   /* Called whenever a service has been resolved successfully or timed out */
   switch (event) {
-  case AVAHI_RESOLVER_FAILURE:
-    debug(3, "(Resolver) Failed to resolve service '%s' of type '%s' in domain '%s': %s.", name,
-          type, domain, avahi_strerror(avahi_client_errno(avahi_service_resolver_get_client(r))));
-    break;
-  case AVAHI_RESOLVER_FOUND: {
-    char a[AVAHI_ADDRESS_STR_MAX], *t;
-    // debug(1, "Resolve callback: Service '%s' of type '%s' in domain '%s':", name, type, domain);
-    char *dacpid = strstr(name, "iTunes_Ctrl_");
-    if (dacpid) {
-      dacpid += strlen("iTunes_Ctrl_");
-      if (strcmp(dacpid, conn->dacp_id) == 0) {
-        if (conn->dacp_port != port) {
+    case AVAHI_RESOLVER_FAILURE:
+      debug(2, "(Resolver) Failed to resolve service '%s' of type '%s' in domain '%s': %s.", name,
+            type, domain, avahi_strerror(avahi_client_errno(avahi_service_resolver_get_client(r))));
+      break;
+    case AVAHI_RESOLVER_FOUND: {
+      //    char a[AVAHI_ADDRESS_STR_MAX], *t;
+      // debug(1, "Resolve callback: Service '%s' of type '%s' in domain '%s':", name, type, domain);
+      char *dacpid = strstr(name, "iTunes_Ctrl_");
+      if (dacpid) {
+        dacpid += strlen("iTunes_Ctrl_");
+        if (strcmp(dacpid, dbs->dacp_id) == 0) {
           debug(3, "Client's DACP port: %u.", port);
-          conn->dacp_port = port;
-#if defined(HAVE_DBUS) || defined(HAVE_MPRIS)
-          set_dacp_server_information(conn);
-#endif
-#ifdef CONFIG_METADATA
+  #ifdef HAVE_DACP_CLIENT
+          dacp_monitor_port_update_callback(dacpid, port);
+  #endif
+  #ifdef CONFIG_METADATA
           char portstring[20];
           memset(portstring, 0, sizeof(portstring));
-          sprintf(portstring, "%u", port);
+          snprintf(portstring, sizeof(portstring), "%u", port);
           send_ssnc_metadata('dapo', strdup(portstring), strlen(portstring), 0);
-#endif
+  #endif
         }
+      } else {
+        debug(1, "Resolve callback: Can't see a DACP string in a DACP Record!");
       }
-    } else {
-      debug(1, "Resolve callback: Can't see a DACP string in a DACP Record!");
     }
   }
-  }
-  avahi_service_resolver_free(r);
+  // debug(1,"service resolver freed by resolve_callback");
+  check_avahi_response(1, avahi_service_resolver_free(r));
 }
 static void browse_callback(AvahiServiceBrowser *b, AvahiIfIndex interface, AvahiProtocol protocol,
                             AvahiBrowserEvent event, const char *name, const char *type,
                             const char *domain, AVAHI_GCC_UNUSED AvahiLookupResultFlags flags,
                             void *userdata) {
-  rtsp_conn_info *conn = (rtsp_conn_info *)userdata;
-  dacp_browser_struct *dbs = (dacp_browser_struct *)conn->mdns_private_pointer;
+  dacp_browser_struct *dbs = (dacp_browser_struct *)userdata;
   assert(b);
   /* Called whenever a new services becomes available on the LAN or is removed from the LAN */
   switch (event) {
@@ -134,22 +140,16 @@ static void browse_callback(AvahiServiceBrowser *b, AvahiIfIndex interface, Avah
     break;
   case AVAHI_BROWSER_REMOVE:
     debug(3, "(Browser) REMOVE: service '%s' of type '%s' in domain '%s'.", name, type, domain);
+#ifdef HAVE_DACP_CLIENT
     char *dacpid = strstr(name, "iTunes_Ctrl_");
     if (dacpid) {
       dacpid += strlen("iTunes_Ctrl_");
-      if (strcmp(dacpid, conn->dacp_id) == 0) {
-        if (conn->dacp_id != 0) {
-          debug(1, "Client's DACP status withdrawn.");
-          conn->dacp_port = 0;
-#if defined(HAVE_DBUS) || defined(HAVE_MPRIS)          
-          set_dacp_server_information(conn); // this will have the effect of telling the scanner that the DACP server is no longer working
-#endif
-        }
-      }
+      if (strcmp(dacpid, dbs->dacp_id) == 0)
+        dacp_monitor_port_update_callback(dbs->dacp_id, 0); // say the port is withdrawn
     } else {
       debug(1, "Browse callback: Can't see a DACP string in a DACP Record!");
     }
-
+#endif
     break;
   case AVAHI_BROWSER_ALL_FOR_NOW:
   case AVAHI_BROWSER_CACHE_EXHAUSTED:
@@ -164,48 +164,52 @@ static void register_service(AvahiClient *c);
 static void egroup_callback(AvahiEntryGroup *g, AvahiEntryGroupState state,
                             AVAHI_GCC_UNUSED void *userdata) {
   switch (state) {
-  case AVAHI_ENTRY_GROUP_ESTABLISHED:
-    /* The entry group has been established successfully */
-    debug(1, "avahi: service '%s' successfully added.", service_name);
-    break;
+    case AVAHI_ENTRY_GROUP_ESTABLISHED:
+      /* The entry group has been established successfully */
+      debug(1, "avahi: service '%s' successfully added.", service_name);
+      break;
 
-  case AVAHI_ENTRY_GROUP_COLLISION: {
-    char *n;
+    case AVAHI_ENTRY_GROUP_COLLISION: {
+      char *n;
 
-    /* A service name collision with a remote service
-     * happened. Let's pick a new name */
-    n = avahi_alternative_service_name(service_name);
-    avahi_free(service_name);
-    service_name = n;
+      /* A service name collision with a remote service
+       * happened. Let's pick a new name */
+      debug(1, "avahi name collision -- look for another");
+      n = avahi_alternative_service_name(service_name);
+      if (service_name)
+        avahi_free(service_name);
+      else
+        debug(1, "avahi attempt to free a NULL service name");
+      service_name = n;
 
-    debug(2, "avahi: service name collision, renaming service to '%s'", service_name);
+      debug(2, "avahi: service name collision, renaming service to '%s'", service_name);
 
-    /* And recreate the services */
-    register_service(avahi_entry_group_get_client(g));
-    break;
-  }
+      /* And recreate the services */
+      register_service(avahi_entry_group_get_client(g));
+      break;
+    }
 
-  case AVAHI_ENTRY_GROUP_FAILURE:
-    debug(1, "avahi: entry group failure: %s",
-          avahi_strerror(avahi_client_errno(avahi_entry_group_get_client(g))));
-    break;
+    case AVAHI_ENTRY_GROUP_FAILURE:
+      debug(1, "avahi: entry group failure: %s",
+            avahi_strerror(avahi_client_errno(avahi_entry_group_get_client(g))));
+      break;
 
-  case AVAHI_ENTRY_GROUP_UNCOMMITED:
-    debug(2, "avahi: service '%s' group is not yet committed.", service_name);
-    break;
+    case AVAHI_ENTRY_GROUP_UNCOMMITED:
+      debug(2, "avahi: service '%s' group is not yet committed.", service_name);
+      break;
 
-  case AVAHI_ENTRY_GROUP_REGISTERING:
-    debug(2, "avahi: service '%s' group is registering.", service_name);
-    break;
+    case AVAHI_ENTRY_GROUP_REGISTERING:
+      debug(2, "avahi: service '%s' group is registering.", service_name);
+      break;
 
-  default:
-    debug(1, "avahi: unhandled egroup state: %d", state);
-    break;
+    default:
+      debug(1, "avahi: unhandled egroup state: %d", state);
+      break;
   }
 }
 
 static void register_service(AvahiClient *c) {
-  debug(1, "avahi: register_service.");
+  // debug(1, "avahi: register_service.");
   if (!group)
     group = avahi_entry_group_new(c, egroup_callback, NULL);
   if (!group)
@@ -227,14 +231,14 @@ static void register_service(AvahiClient *c) {
                                           service_name, config.regtype, NULL, NULL, port,
                                           MDNS_RECORD_WITH_METADATA, NULL);
       if (ret == 0)
-        debug(1, "avahi: request to add \"%s\" service with metadata", config.regtype);
+        debug(2, "avahi: request to add \"%s\" service with metadata", config.regtype);
     } else {
 #endif
       ret = avahi_entry_group_add_service(group, selected_interface, AVAHI_PROTO_UNSPEC, 0,
                                           service_name, config.regtype, NULL, NULL, port,
                                           MDNS_RECORD_WITHOUT_METADATA, NULL);
       if (ret == 0)
-        debug(1, "avahi: request to add \"%s\" service without metadata", config.regtype);
+        debug(2, "avahi: request to add \"%s\" service without metadata", config.regtype);
 #ifdef CONFIG_METADATA
     }
 #endif
@@ -256,7 +260,7 @@ static void client_callback(AvahiClient *c, AvahiClientState state,
   switch (state) {
   case AVAHI_CLIENT_S_REGISTERING:
     if (group)
-      avahi_entry_group_reset(group);
+      check_avahi_response(1, avahi_entry_group_reset(group));
     break;
 
   case AVAHI_CLIENT_S_RUNNING:
@@ -268,8 +272,12 @@ static void client_callback(AvahiClient *c, AvahiClientState state,
     debug(1, "avahi: client failure: %s", avahi_strerror(err));
 
     if (err == AVAHI_ERR_DISCONNECTED) {
+      debug(1, "avahi client -- we have been disconnected, so let's reconnect.");
       /* We have been disconnected, so lets reconnect */
-      avahi_client_free(c);
+      if (c)
+        avahi_client_free(c);
+      else
+        debug(1, "Attempt to free NULL avahi client");
       c = NULL;
       group = NULL;
 
@@ -301,8 +309,7 @@ static void client_callback(AvahiClient *c, AvahiClientState state,
 static void service_client_callback(AvahiClient *c, AvahiClientState state, void *userdata) {
   int err;
 
-  rtsp_conn_info *conn = (rtsp_conn_info *)userdata;
-  dacp_browser_struct *dbs = (dacp_browser_struct *)conn->mdns_private_pointer;
+  dacp_browser_struct *dbs = (dacp_browser_struct *)userdata;
 
   switch (state) {
   case AVAHI_CLIENT_S_REGISTERING:
@@ -316,8 +323,12 @@ static void service_client_callback(AvahiClient *c, AvahiClientState state, void
     debug(1, "avahi: service client failure: %s", avahi_strerror(err));
 
     if (err == AVAHI_ERR_DISCONNECTED) {
+      debug(1, "avahi client has been disconnected, so reconnect");
       /* We have been disconnected, so lets reconnect */
-      avahi_client_free(c);
+      if (c)
+        avahi_client_free(c);
+      else
+        debug(1, "avahi attempt to free a NULL client");
       c = NULL;
 
       if (!(dbs->service_client =
@@ -348,7 +359,7 @@ static void service_client_callback(AvahiClient *c, AvahiClientState state, void
 }
 
 static int avahi_register(char *srvname, int srvport) {
-  debug(1, "avahi: avahi_register.");
+  // debug(1, "avahi: avahi_register.");
   service_name = strdup(srvname);
   port = srvport;
 
@@ -373,95 +384,110 @@ static int avahi_register(char *srvname, int srvport) {
 
 static void avahi_unregister(void) {
   debug(1, "avahi: avahi_unregister.");
-  if (tpoll)
+  if (tpoll) {
     avahi_threaded_poll_stop(tpoll);
-  tpoll = NULL;
+
+    if (client) {
+      avahi_client_free(client);
+      client = NULL;
+    } else {
+      debug(1, "avahi attempting to unregister a NULL client");
+    }
+    avahi_threaded_poll_free(tpoll);
+    tpoll = NULL;
+  }
 
   if (service_name)
     free(service_name);
+  else
+    debug(1, "avahi attempt to free NULL service name");
   service_name = NULL;
 }
 
-int avahi_dacp_monitor(rtsp_conn_info *conn) {
-
+void *avahi_dacp_monitor(char *dacp_id) {
   dacp_browser_struct *dbs = (dacp_browser_struct *)malloc(sizeof(dacp_browser_struct));
-  
 
   if (dbs == NULL)
     die("can not allocate a dacp_browser_struct.");
-    
-  conn->mdns_private_pointer = (void *)dbs;
+
+  char *t = strdup(dacp_id);
+  if (t)
+    dbs->dacp_id = t;
+  else {
+    die("can not allocate a dacp_id string in dacp_browser_struct.");
+  }
 
   // create the threaded poll code
   int err;
   if (!(dbs->service_poll = avahi_threaded_poll_new())) {
     warn("couldn't create avahi threaded service_poll!");
-    if (dbs) {
-      free((char *)dbs);
-    }
-    conn->mdns_private_pointer = NULL;
-    return -1;
+    free(dbs->dacp_id);
+    free((char *)dbs);
+    return NULL;
   }
 
   // create the service client
   if (!(dbs->service_client =
             avahi_client_new(avahi_threaded_poll_get(dbs->service_poll), AVAHI_CLIENT_NO_FAIL,
-                             service_client_callback, (void *)conn, &err))) {
+                             service_client_callback, (void *)dbs, &err))) {
     warn("couldn't create avahi service client: %s!", avahi_strerror(err));
-    if (dbs) { // should free the threaded poll code
-      avahi_threaded_poll_free(dbs->service_poll);
-      free((char *)dbs);
-    }
-    conn->mdns_private_pointer = NULL;
-    return -1;
+    avahi_threaded_poll_free(dbs->service_poll);
+    free(dbs->dacp_id);
+    free((char *)dbs);
+    return NULL;
   }
 
   /* Create the service browser */
   if (!(dbs->service_browser =
             avahi_service_browser_new(dbs->service_client, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC,
-                                      "_dacp._tcp", NULL, 0, browse_callback, (void *)conn))) {
-    warn("Failed to create service browser: %s\n",
+                                      "_dacp._tcp", NULL, 0, browse_callback, (void *)dbs))) {
+    warn("failed to create avahi service browser: %s\n",
          avahi_strerror(avahi_client_errno(dbs->service_client)));
-    if (dbs) { // should free the threaded poll code and the service client
-      avahi_client_free(dbs->service_client);
-      avahi_threaded_poll_free(dbs->service_poll);
-      free((char *)dbs);
-    }
-    conn->mdns_private_pointer = NULL;
-    return -1;
+    avahi_client_free(dbs->service_client);
+    avahi_threaded_poll_free(dbs->service_poll);
+    free(dbs->dacp_id);
+    free((char *)dbs);
+    return NULL;
   }
   // start the polling thread
   if (avahi_threaded_poll_start(dbs->service_poll) < 0) {
     warn("couldn't start avahi service_poll thread");
-    if (dbs) { // should free the threaded poll code and the service client and the service browser
-      avahi_service_browser_free(dbs->service_browser);
-      avahi_client_free(dbs->service_client);
-      avahi_threaded_poll_free(dbs->service_poll);
-      free((char *)dbs);
-    }
-    conn->mdns_private_pointer = NULL;
-    return -1;
+    avahi_service_browser_free(dbs->service_browser);
+    avahi_client_free(dbs->service_client);
+    avahi_threaded_poll_free(dbs->service_poll);
+    free(dbs->dacp_id);
+    free((char *)dbs);
+    return NULL;
   }
-  return 0;
+  debug(3, "Avahi DACP monitor successfully started");
+  return (void *)dbs;
 }
 
-void avahi_dacp_dont_monitor(rtsp_conn_info *conn) {
-  dacp_browser_struct *dbs = (dacp_browser_struct *)conn->mdns_private_pointer;
-  if (dbs) {
+void avahi_dacp_dont_monitor(void *userdata) {
+  debug(3,"avahi_dacp_dont_monitor");
+  if (userdata) {
+    dacp_browser_struct *dbs = (dacp_browser_struct *)userdata;
     // stop and dispose of everything
-    if ((dbs)->service_poll)
+    /*if (dbs->service_poll)
       avahi_threaded_poll_stop((dbs)->service_poll);
-    if ((dbs)->service_browser)
-      avahi_service_browser_free((dbs)->service_browser);
-    if ((dbs)->service_client)
-      avahi_client_free((dbs)->service_client);
-    if ((dbs)->service_poll)
-      avahi_threaded_poll_free((dbs)->service_poll);
-    free((char *)(dbs));
-    conn->mdns_private_pointer = NULL;
+    */
+    if (dbs->service_poll) {
+      avahi_threaded_poll_stop(dbs->service_poll);     
+      avahi_threaded_poll_lock(dbs->service_poll);
+      if (dbs->service_browser)
+        avahi_service_browser_free(dbs->service_browser);
+      if (dbs->service_client)
+        avahi_client_free(dbs->service_client);
+      avahi_threaded_poll_unlock(dbs->service_poll); 
+      avahi_threaded_poll_free(dbs->service_poll);
+    }
+    free(dbs->dacp_id);
+    free(userdata);
+    debug(3, "Avahi DACP monitor successfully stopped");
   } else {
-    debug(1, "DHCP Monitor is not running.");
+    debug(1, "Avahi DACP Monitor is not running.");
   }
+  debug(3,"avahi_dacp_dont_monitor exit");
 }
 
 mdns_backend mdns_avahi = {.name = "avahi",
